searchData={"content_type":"text/markdown","items":[{"doc":"Module for making and processing AWS requests.","ref":"ExAws.html","title":"ExAws","type":"module"},{"doc":"Perform an AWS request.\n\nFirst build an operation from one of the services, and then pass it to this\nfunction to perform it.\n\nIf you want to build an operation manually, see: `ExAws.Operation`\n\nThis function takes an optional second parameter of configuration overrides.\nThis is useful if you want to have certain configuration changed on a per\nrequest basis.\n\nAlso you can configure telemetry metrics with:\n\n  * `:telemetry_event` - The telemetry event name to dispatch the event under.\n    Defaults to `[:ex_aws, :request]`.\n  * `:telemetry_options` - Extra options to attach to telemetry event name.","ref":"ExAws.html#request/2","title":"ExAws.request/2","type":"function"},{"doc":"If you have one of the service modules installed, you can just use those service\nmodules like this:\n\n    ExAws.S3.list_buckets |> ExAws.request\n\n    ExAws.S3.list_buckets |> ExAws.request(region: \"eu-west-1\")\n\n    ExAws.Dynamo.get_object(\"users\", \"foo@bar.com\") |> ExAws.request\n\nAlternatively you can create operation structs manually for services\nthat aren't supported:\n\n    op = %ExAws.Operation.JSON{\n      http_method: :post,\n      service: :dynamodb,\n      headers: [\n        {\"x-amz-target\", \"DynamoDB_20120810.ListTables\"},\n        {\"content-type\", \"application/x-amz-json-1.0\"}\n      ],\n    }\n\n    ExAws.request(op)","ref":"ExAws.html#request/2-examples","title":"Examples - ExAws.request/2","type":"function"},{"doc":"The following events are published:\n\n* `[:ex_aws, :request, :start]` - dispatched on start every request sent to the AWS.\n* `[:ex_aws, :request, :stop]` - dispatched on every response from AWS.\n* `[:ex_aws, :request, :exception]` - dispatched after exceptions on request sent to AWS.\n\nWith `:metadata` map including the following fields:\n\n  * `:result` - the request result: `:ok` or `:error`\n  * `:attempt` - the attempt number\n  * `:service` - the AWS service\n  * `:options` - extra options given to the repo operation under\n    `:telemetry_options`","ref":"ExAws.html#request/2-telemetry-events","title":"Telemetry events - ExAws.request/2","type":"function"},{"doc":"Perform an AWS request, raise if it fails.\n\nSame as `request/1,2` except it will either return the successful response from\nAWS or raise an exception.","ref":"ExAws.html#request!/2","title":"ExAws.request!/2","type":"function"},{"doc":"Return a stream for the AWS resource.","ref":"ExAws.html#stream!/2","title":"ExAws.stream!/2","type":"function"},{"doc":"ExAws.S3.list_objects(\"my-bucket\") |> ExAws.stream!","ref":"ExAws.html#stream!/2-examples","title":"Examples - ExAws.stream!/2","type":"function"},{"doc":"A behaviour definition for the core operations of ExAws.\n\n`ExAws` implements this behaviour.","ref":"ExAws.Behaviour.html","title":"ExAws.Behaviour","type":"behaviour"},{"doc":"See `ExAws.request/2`.","ref":"ExAws.Behaviour.html#c:request/1","title":"ExAws.Behaviour.request/1","type":"callback"},{"doc":"See `ExAws.request/2`.","ref":"ExAws.Behaviour.html#c:request/2","title":"ExAws.Behaviour.request/2","type":"callback"},{"doc":"See `ExAws.request!/2`.","ref":"ExAws.Behaviour.html#c:request!/1","title":"ExAws.Behaviour.request!/1","type":"callback"},{"doc":"See `ExAws.request!/2`.","ref":"ExAws.Behaviour.html#c:request!/2","title":"ExAws.Behaviour.request!/2","type":"callback"},{"doc":"See `ExAws.stream!/2`.","ref":"ExAws.Behaviour.html#c:stream!/1","title":"ExAws.Behaviour.stream!/1","type":"callback"},{"doc":"See `ExAws.stream!/2`.","ref":"ExAws.Behaviour.html#c:stream!/2","title":"ExAws.Behaviour.stream!/2","type":"callback"},{"doc":"Generates the configuration for a service.\n\nIt starts with the defaults for a given environment and then merges in the\ncommon config from the ex_aws config root, and then finally any config\nspecified for the particular service.","ref":"ExAws.Config.html","title":"ExAws.Config","type":"module"},{"doc":"Some fields are marked as refreshable. These fields will be fetched through\nthe auth cache even if they are passed in as overrides. This is so stale\ncredentials aren't used, for example, with long running streams.\n\nThis behaviour must be explicitly enabled by passing `refreshable: true` as an option\nto Config.new/2","ref":"ExAws.Config.html#module-refreshable-fields","title":"Refreshable fields - ExAws.Config","type":"module"},{"doc":"","ref":"ExAws.Config.html#awscli_auth_adapter/0","title":"ExAws.Config.awscli_auth_adapter/0","type":"function"},{"doc":"","ref":"ExAws.Config.html#awscli_auth_credentials/2","title":"ExAws.Config.awscli_auth_credentials/2","type":"function"},{"doc":"","ref":"ExAws.Config.html#build_base/2","title":"ExAws.Config.build_base/2","type":"function"},{"doc":"Builds a minimal HTTP configuration.","ref":"ExAws.Config.html#http_config/2","title":"ExAws.Config.http_config/2","type":"function"},{"doc":"Builds a complete set of config for an operation.\n\n  1. Defaults are pulled from `ExAws.Config.Defaults`\n  2. Common values set via e.g `config :ex_aws` are merged in.\n  3. Keys set on the individual service e.g `config :ex_aws, :s3` are merged in\n  4. Finally, any configuration overrides are merged in","ref":"ExAws.Config.html#new/2","title":"ExAws.Config.new/2","type":"function"},{"doc":"","ref":"ExAws.Config.html#parse_host_for_region/1","title":"ExAws.Config.parse_host_for_region/1","type":"function"},{"doc":"","ref":"ExAws.Config.html#retrieve_runtime_config/1","title":"ExAws.Config.retrieve_runtime_config/1","type":"function"},{"doc":"","ref":"ExAws.Config.html#retrieve_runtime_value/2","title":"ExAws.Config.retrieve_runtime_value/2","type":"function"},{"doc":"","ref":"ExAws.Config.html#t:t/0","title":"ExAws.Config.t/0","type":"type"},{"doc":"Defaults for each service","ref":"ExAws.Config.Defaults.html","title":"ExAws.Config.Defaults","type":"module"},{"doc":"","ref":"ExAws.Config.Defaults.html#apply_defaults/4","title":"ExAws.Config.Defaults.apply_defaults/4","type":"function"},{"doc":"Retrieve the default configuration for a service.","ref":"ExAws.Config.Defaults.html#defaults/1","title":"ExAws.Config.Defaults.defaults/1","type":"function"},{"doc":"","ref":"ExAws.Config.Defaults.html#get/2","title":"ExAws.Config.Defaults.get/2","type":"function"},{"doc":"","ref":"ExAws.Config.Defaults.html#host/2","title":"ExAws.Config.Defaults.host/2","type":"function"},{"doc":"Specifies expected behaviour of a credentials provider.\n\nA credentials initializer provider is a module that fetches the AWS credentials from different sources.","ref":"ExAws.CredentialsIni.Provider.html","title":"ExAws.CredentialsIni.Provider","type":"behaviour"},{"doc":"","ref":"ExAws.CredentialsIni.Provider.html#c:security_credentials/1","title":"ExAws.CredentialsIni.Provider.security_credentials/1","type":"callback"},{"doc":"","ref":"ExAws.CredentialsIni.Provider.html#t:credentials/0","title":"ExAws.CredentialsIni.Provider.credentials/0","type":"type"},{"doc":"","ref":"ExAws.CredentialsIni.Provider.html#t:profile/0","title":"ExAws.CredentialsIni.Provider.profile/0","type":"type"},{"doc":"For use with IMDSv2, this module retrieves the metadata session token and refreshes it before expiration.","ref":"ExAws.InstanceMetaTokenProvider.html","title":"ExAws.InstanceMetaTokenProvider","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExAws.InstanceMetaTokenProvider.html#child_spec/1","title":"ExAws.InstanceMetaTokenProvider.child_spec/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#get/1","title":"ExAws.InstanceMetaTokenProvider.get/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#get_headers/1","title":"ExAws.InstanceMetaTokenProvider.get_headers/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#init/1","title":"ExAws.InstanceMetaTokenProvider.init/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#refresh_in/1","title":"ExAws.InstanceMetaTokenProvider.refresh_in/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#refresh_token/2","title":"ExAws.InstanceMetaTokenProvider.refresh_token/2","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#request_token/1","title":"ExAws.InstanceMetaTokenProvider.request_token/1","type":"function"},{"doc":"","ref":"ExAws.InstanceMetaTokenProvider.html#start_link/1","title":"ExAws.InstanceMetaTokenProvider.start_link/1","type":"function"},{"doc":"Defines the specification for a JSON codec.\n\nExAws supports the use of your favorite JSON codec provided it fulfills this specification.\nPoison fulfills this spec without modification, and is the default.\n\nSee the contents of `ExAws.JSON.JSX` for an example of an alternative implementation.","ref":"ExAws.JSON.Codec.html","title":"ExAws.JSON.Codec","type":"behaviour"},{"doc":"Here for example is the code required to make jsx comply with this spec.\n\nIn your config you would do:\n\n    config :ex_aws,\n      json_codec: ExAws.JSON.JSX\n\n    defmodule ExAws.JSON.JSX do\n      @behaviour ExAws.JSON.Codec\n\n      @moduledoc false\n\n      def encode!(%{} = map) do\n        map |> :jsx.encode\n      end\n\n      def encode(map) do\n        try do\n          {:ok, encode!(map)}\n        rescue\n          ArgumentError -> {:error, :badarg}\n        end\n      end\n\n      def decode!(string) do\n        :jsx.decode(string, [:return_maps])\n      end\n\n      def decode(string) do\n        try do\n          {:ok, decode!(string)}\n        rescue\n          ArgumentError -> {:error, :badarg}\n        end\n      end\n    end","ref":"ExAws.JSON.Codec.html#module-example","title":"Example - ExAws.JSON.Codec","type":"behaviour"},{"doc":"","ref":"ExAws.JSON.Codec.html#c:decode/1","title":"ExAws.JSON.Codec.decode/1","type":"callback"},{"doc":"","ref":"ExAws.JSON.Codec.html#c:decode!/1","title":"ExAws.JSON.Codec.decode!/1","type":"callback"},{"doc":"","ref":"ExAws.JSON.Codec.html#c:encode/1","title":"ExAws.JSON.Codec.encode/1","type":"callback"},{"doc":"","ref":"ExAws.JSON.Codec.html#c:encode!/1","title":"ExAws.JSON.Codec.encode!/1","type":"callback"},{"doc":"An operation to perform on AWS.\n\nThis module defines a protocol for executing operations on AWS. ExAws ships with\nseveral different modules that each implement the `ExAws.Operation` protocol. These\nmodules each handle one of the broad categories of AWS service types:\n\n- `ExAws.Operation.JSON`\n- `ExAws.Operation.Query`\n- `ExAws.Operation.RestQuery`\n- `ExAws.Operation.S3`\n\nExAws works by creating a data structure that implements this protocol, and then\ncalling `perform/2` on it.","ref":"ExAws.Operation.html","title":"ExAws.Operation","type":"protocol"},{"doc":"Perform a request on AWS.\n\nThe operation is synchronous, returning a response or an error.","ref":"ExAws.Operation.html#perform/2","title":"ExAws.Operation.perform/2","type":"function"},{"doc":"%ExAws.Operation.JSON{\n      data: %{},\n      headers: [\n        {\"x-amz-target\", \"DynamoDB_20120810.ListTables\"},\n        {\"content-type\", \"application/x-amz-json-1.0\"}\n      ],\n      http_method: :post,\n      params: %{},\n      path: \"/\",\n      service: :dynamodb,\n    } |> ExAws.Operation.perform(ExAws.Config.new(:dynamodb))","ref":"ExAws.Operation.html#perform/2-example","title":"Example - ExAws.Operation.perform/2","type":"function"},{"doc":"Perform a *streaming* request on AWS.","ref":"ExAws.Operation.html#stream!/2","title":"ExAws.Operation.stream!/2","type":"function"},{"doc":"All the types that implement this protocol.","ref":"ExAws.Operation.html#t:t/0","title":"ExAws.Operation.t/0","type":"type"},{"doc":"Datastructure representing an operation on a JSON based AWS service.\n\nThis module is generally not used directly, but rather is constructed by one\nof the relevant AWS services.\n\nThese include:\n- DynamoDB\n- Kinesis\n- Lambda (Rest style)\n- ElasticTranscoder\n\nJSON services are generally pretty simple. You just need to populate the `data`\nattribute with whatever request body parameters need converted to JSON, and set\nany service specific headers.\n\nThe `before_request`","ref":"ExAws.Operation.JSON.html","title":"ExAws.Operation.JSON","type":"module"},{"doc":"","ref":"ExAws.Operation.JSON.html#new/2","title":"ExAws.Operation.JSON.new/2","type":"function"},{"doc":"","ref":"ExAws.Operation.JSON.html#t:t/0","title":"ExAws.Operation.JSON.t/0","type":"type"},{"doc":"Datastructure representing an operation on a Query based AWS service\n\nThese include:\n- SQS\n- SNS\n- SES","ref":"ExAws.Operation.Query.html","title":"ExAws.Operation.Query","type":"module"},{"doc":"","ref":"ExAws.Operation.Query.html#t:t/0","title":"ExAws.Operation.Query.t/0","type":"type"},{"doc":"Holds data necessary for an operation on the S3 service.","ref":"ExAws.Operation.S3.html","title":"ExAws.Operation.S3","type":"module"},{"doc":"","ref":"ExAws.Operation.S3.html#t:t/0","title":"ExAws.Operation.S3.t/0","type":"type"},{"doc":"Configuration for `:hackney`.\n\nOptions can be set for `:hackney` with the following config:\n\n    config :ex_aws, :hackney_opts,\n      recv_timeout: 30_000\n\nThe default config handles setting the above.","ref":"ExAws.Request.Hackney.html","title":"ExAws.Request.Hackney","type":"module"},{"doc":"","ref":"ExAws.Request.Hackney.html#request/5","title":"ExAws.Request.Hackney.request/5","type":"function"},{"doc":"Specifies expected behaviour of an HTTP client.\n\nExAws allows you to use your HTTP client of choice, provided that\nit can be coerced into complying with this module's specification.\n\nThe default is `:hackney`.","ref":"ExAws.Request.HttpClient.html","title":"ExAws.Request.HttpClient","type":"behaviour"},{"doc":"Here is an example using [Req](https://hexdocs.pm/req/readme.html).\n\nFirst, create a module implementing the `ExAws.Request.HttpClient` behaviour.\n\n```\ndefmodule ExAws.Request.Req do\n  @moduledoc \"\"\"\n  ExAws HTTP client implementation for Req.\n  \"\"\"\n  @behaviour ExAws.Request.HttpClient\n\n  @impl ExAws.Request.HttpClient\n  def request(method, url, body, headers, _http_opts) do\n    case Req.request(method: method, url: url, body: body, headers: headers, decode_body: false) do\n      {:ok, response} -> {:ok, adapt_response(response)}\n      {:error, reason} -> {:error, %{reason: reason}}\n    end\n  end\n\n  defp adapt_response(response) do\n    # adapt the response to fit the shape expected by ExAWS\n    flat_headers =\n      Enum.flat_map(response.headers, fn\n        {name, vals} when is_list(vals) -> Enum.map(vals, &{name, &1})\n        {name, val} -> {name, val}\n      end)\n\n    %{\n      body: response.body,\n      status_code: response.status,\n      headers: flat_headers\n    }\n  end\nend\n```\n\nThen, in build-time config (e.g. config.exs):\n\n```\nconfig :ex_aws,\n  http_client: ExAws.Request.Req\n```\n\nWhen conforming your selected HTTP Client take note of a few things:\n\n  - The module name doesn't need to follow the same styling as this module it\n    is simply your own 'HTTP Client', i.e. `MyApp.HttpClient`\n\n  - The request function must accept the methods as described in the\n    `c:request/5` callback, you can however set these as optional,\n    i.e. `http_opts \\ []`\n\n  - Ensure the call to your chosen HTTP Client is correct and the return is\n    in the same format as defined in the `c:request/5` callback","ref":"ExAws.Request.HttpClient.html#module-example-req","title":"Example: Req - ExAws.Request.HttpClient","type":"behaviour"},{"doc":"def request(method, url, body, headers, http_opts \\ []) do\n      Mojito.request(method, url, headers, body, http_opts)\n    end","ref":"ExAws.Request.HttpClient.html#module-example-mojito","title":"Example: Mojito - ExAws.Request.HttpClient","type":"behaviour"},{"doc":"","ref":"ExAws.Request.HttpClient.html#c:request/5","title":"ExAws.Request.HttpClient.request/5","type":"callback"},{"doc":"","ref":"ExAws.Request.HttpClient.html#t:http_method/0","title":"ExAws.Request.HttpClient.http_method/0","type":"type"},{"doc":"A generic AWS error.","ref":"ExAws.Error.html","title":"ExAws.Error","type":"exception"},{"doc":"# ExAws\n\n \n\n![GitHub Workflow Status](https://img.shields.io/github/workflow/status/ex-aws/ex_aws/on-push)\n[![hex.pm](https://img.shields.io/hexpm/v/ex_aws.svg)](https://hex.pm/packages/ex_aws)\n[![hex.pm](https://img.shields.io/hexpm/dt/ex_aws.svg)](https://hex.pm/packages/ex_aws)\n[![hex.pm](https://img.shields.io/hexpm/l/ex_aws.svg)](https://hex.pm/packages/ex_aws)\n[![hexdocs.pm](https://img.shields.io/badge/hexdocs-release-lightgreen.svg)](https://hexdocs.pm/ex_aws)\n[![github.com](https://img.shields.io/github/last-commit/ex-aws/ex_aws.svg)](https://github.com/ex-aws/ex_aws/commits/master)\n\nA flexible easy to use set of AWS APIs.\n\nAvailable Services: https://github.com/ex-aws?q=service&type=&language=","ref":"readme.html","title":"ExAws","type":"extras"},{"doc":"ExAws v2.0 breaks out every service into its own package. To use the S3\nservice, you need both the core `:ex_aws` package as well as the `:ex_aws_s3`\npackage.\n\nAs with all ExAws services, you'll need a compatible HTTP client (defaults to\n`:hackney`) and whatever JSON or XML codecs needed by the services you want to\nuse. Consult individual service documentation for details on what each service\nneeds.\n\n```elixir\ndefp deps do\n  [\n    {:ex_aws, \"~> 2.1\"},\n    {:ex_aws_s3, \"~> 2.0\"},\n    {:hackney, \"~> 1.9\"},\n    {:sweet_xml, \"~> 0.6\"},\n  ]\nend\n```\n\nWith these deps you can use `ExAws` precisely as you're used to:\n\n```\n# make a request (with the default region)\nExAws.S3.list_objects(\"my-bucket\") |> ExAws.request()\n\n# or specify the region\nExAws.S3.list_objects(\"my-bucket\") |> ExAws.request(region: \"us-west-1\")\n\n# some operations support streaming\nExAws.S3.list_objects(\"my-bucket\") |> ExAws.stream!() |> Enum.to_list()\n```\n\n#","ref":"readme.html#getting-started","title":"Getting Started - ExAws","type":"extras"},{"doc":"ExAws requires valid AWS keys in order to work properly. ExAws by default does\nthe equivalent of:\n\n```elixir\nconfig :ex_aws,\n  access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, :instance_role],\n  secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, :instance_role]\n```\n\nThis means it will try to resolve credentials in order:\n\n* Look for the AWS standard `AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY` environment variables\n* Resolve credentials with IAM\n  * If running inside ECS and a [task role](https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html) has been assigned it will use it\n  * Otherwise it will fall back to the [instance role](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html)\n\nAWS CLI config files are supported, but require an additional dependency:\n\n```elixir\n{:configparser_ex, \"~> 4.0\"}\n```\n\nYou can then add `{:awscli, \"profile_name\", timeout}` to the above config and\nit will pull information from `~/.aws/config` and `~/.aws/credentials`\n\nAlternatively, if you already have a profile name set in the `AWS_PROFILE` environment\nvariable, you can use that with `{:awscli, :system, timeout}`\n\n```elixir\nconfig :ex_aws,\n  access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, {:awscli, \"default\", 30}, :instance_role],\n  secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, {:awscli, \"default\", 30}, :instance_role]\n```\n\nFor role based authentication via `role_arn` and `source_profile` an additional\ndependency is required:\n\n```elixir\n{:ex_aws_sts, \"~> 2.0\"}\n```\n\nFurther information on role based authentication is provided in said dependency.\n\n##","ref":"readme.html#aws-key-configuration","title":"AWS Key configuration - ExAws","type":"extras"},{"doc":"Alternatively, you can also provide `AWS_SESSION_TOKEN` to `security_token` to authenticate\nwith session token:\n\n```elixir\nconfig :ex_aws,\n  access_key_id: {:system, \"AWS_ACCESS_KEY_ID\"},\n  security_token: {:system, \"AWS_SESSION_TOKEN\"},\n  secret_access_key: {:system, \"AWS_SECRET_ACCESS_KEY\"}\n```\n\n#","ref":"readme.html#session-token-configuration","title":"Session token configuration - ExAws","type":"extras"},{"doc":"ExAws by default uses [hackney](https://github.com/benoitc/hackney) to make\nHTTP requests to AWS API. You can modify the options as such:\n\n```elixir\nconfig :ex_aws, :hackney_opts,\n  follow_redirect: true,\n  recv_timeout: 30_000\n```\n\n#","ref":"readme.html#hackney-configuration","title":"Hackney configuration - ExAws","type":"extras"},{"doc":"You can set the region used by default for requests.\n\n```elixir\nconfig :ex_aws,\n  region: \"us-west-2\",\n```\n\nAlternatively, the region can be set in an environment variable:\n\n```elixir\nconfig :ex_aws,\n  region: {:system, \"AWS_REGION\"}\n```\n\n#","ref":"readme.html#aws-region-configuration","title":"AWS Region Configuration. - ExAws","type":"extras"},{"doc":"The default JSON codec is Jason.  You can choose a different one:\n\n```elixir\nconfig :ex_aws,\n  json_codec: Poison\n```\n\n#","ref":"readme.html#json-codec-configuration","title":"JSON Codec Configuration - ExAws","type":"extras"},{"doc":"Paths that include multiple consecutive /'s will by default be normalized\nto a single slash. There are cases when paths need to be literal (S3) and\nthis normalization behaviour can be turned off via configuration:\n\n```elixir\nconfig :ex_aws,\n  normalize_path: false\n```","ref":"readme.html#path-normalization","title":"Path Normalization - ExAws","type":"extras"},{"doc":"ExAws can also be used directly without any specific service module.\n\nYou need to figure out how the API of the specific AWS service works, in particular:\n\n- Protocol (JSON or query).\n- Path (depends on the service and the specific operation, usually \"/\").\n- Service name (used to generate the request signature,\n  [as described here](https://docs.aws.amazon.com/general/latest/gr/sigv4-create-string-to-sign.html)).\n- Request body, query params, HTTP method, and headers (depends on the service\n  and specific operation).\n\nYou can look for this information in the service's API reference at\n[docs.aws.amazon.com](https://docs.aws.amazon.com/index.html) or, for example,\nin the Go SDK API models at [github.com/aws/aws-sdk-go](https://github.com/aws/aws-sdk-go/tree/master/models/apis) (look for a `api-*.json` file).\n\nThe protocol dictates which operation module to use for the request. If the\nprotocol is JSON, use `ExAws.Operation.JSON`, if it's query, use\n`ExAws.Operation.Query`.\n\n#","ref":"readme.html#direct-usage","title":"Direct Usage - ExAws","type":"extras"},{"doc":"##","ref":"readme.html#examples","title":"Examples - ExAws","type":"extras"},{"doc":"```elixir\naction = :describe_clusters\naction_string = action |> Atom.to_string |> Macro.camelize\n\noperation =\n  %ExAws.Operation.Query{\n    path: \"/\",\n    params: %{\"Action\" => action_string},\n    service: :redshift,\n    action: action\n  }\n\nExAws.request(operation)\n```\n\n##","ref":"readme.html#redshift-describeclusters","title":"Redshift DescribeClusters - ExAws","type":"extras"},{"doc":"```elixir\ndata = %{\n  taskDefinition: \"hello_world\",\n  launchType: \"FARGATE\",\n  networkConfiguration: %{\n    awsvpcConfiguration: %{\n      subnets: [\"subnet-1a2b3c4d\", \"subnet-4d3c2b1a\"],\n      securityGroups: [\"sg-1a2b3c4d\"],\n      assignPublicIp: \"ENABLED\"\n    }\n  }\n}\n\noperation =\n  %ExAws.Operation.JSON{\n    http_method: :post,\n    headers: [\n      {\"x-amz-target\", \"AmazonEC2ContainerServiceV20141113.RunTask\"},\n      {\"content-type\", \"application/x-amz-json-1.1\"}\n    ],\n    path: \"/\",\n    data: data,\n    service: :ecs\n}\n\nExAws.request(operation)\n```","ref":"readme.html#ecs-runtask","title":"ECS RunTask - ExAws","type":"extras"},{"doc":"- Easy configuration.\n- Minimal dependencies. Choose your favorite JSON codec and HTTP client.\n- Elixir streams to automatically retrieve paginated resources.\n- Elixir protocols allow easy customization of Dynamo encoding / decoding.\n- Simple. ExAws aims to provide a clear and consistent elixir wrapping around\n  AWS APIs, not abstract them away entirely. For every action in a given AWS\n  API there is a corresponding function within the appropriate module. Higher\n  level abstractions like the aforementioned streams are in addition to and not\n  instead of basic API calls.\n\nThat's it!","ref":"readme.html#highlighted-features","title":"Highlighted Features - ExAws","type":"extras"},{"doc":"ExAws will retry failed AWS API requests using exponential backoff per the\n\"Full Jitter\" formula described in\nhttps://www.awsarchitectureblog.com/2015/03/backoff.html\n\nThe algorithm uses three values, which are configurable:\n\n```elixir\n# default values shown below\n\nconfig :ex_aws, :retries,\n  max_attempts: 10,\n  base_backoff_in_ms: 10,\n  max_backoff_in_ms: 10_000\n```\n\n* `max_attempts` is the maximum number of possible attempts with backoffs in between each one\n* `base_backoff_in_ms` corresponds to the `base` value described in the blog post\n* `max_backoff_in_ms` corresponds to the `cap` value described in the blog post","ref":"readme.html#retries","title":"Retries - ExAws","type":"extras"},{"doc":"If you want to run `mix test`, you'll need to have a local `dynamodb` running\non port 8000.  See [Setting up DynamoDB Local](https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DynamoDBLocal.DownloadingAndRunning.html).\n\nThe redirect test will intentionally cause a warning to be issued.","ref":"readme.html#testing","title":"Testing - ExAws","type":"extras"},{"doc":"The MIT License (MIT)\n\nCopyright (c) 2014-2020 CargoSense, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"readme.html#license","title":"License - ExAws","type":"extras"}]}